@@ -12,23 +12,76 @@
 #
 """``scenedetect.frame_timecode`` Module
 
-This module contains the :py:class:`FrameTimecode` object, which is used as a way for
-PySceneDetect to store frame-accurate timestamps of each cut. This is done by also
-specifying the video framerate with the timecode, allowing a frame number to be
-converted to/from a floating-point number of seconds, or string in the form
-`"HH:MM:SS[.nnn]"` where the `[.nnn]` part is optional.
+This module contains the :py:class:`FrameTimecode` object, which is used as a way for PySceneDetect
+to store frame-accurate timestamps of each cut. This is done by also specifying the video framerate
+with the timecode, allowing a frame number to be converted to/from a floating-point number of
+seconds, or string in the form `"HH:MM:SS[.nnn]"` where the `[.nnn]` part is optional.
 
 See the following examples, or the :py:class:`FrameTimecode constructor <FrameTimecode>`.
 
-Unit tests for the FrameTimecode object can be found in `tests/test_timecode.py`.
+===============================================================
+Usage Examples
+===============================================================
+
+A :py:class:`FrameTimecode` can be created by specifying a timecode (`int` for number of frames,
+`float` for number of seconds, or `str` in the form "HH:MM:SS" or "HH:MM:SS.nnn") with a framerate:
+
+.. code:: python
+
+    frames = FrameTimecode(timecode = 29, fps = 29.97)
+    seconds_float = FrameTimecode(timecode = 10.0, fps = 10.0)
+    timecode_str = FrameTimecode(timecode = "00:00:10.000", fps = 10.0)
+
+
+Arithmetic/comparison operations with :py:class:`FrameTimecode` objects is also possible, and the
+other operand can also be of the above types:
+
+.. code:: python
+
+    x = FrameTimecode(timecode = "00:01:00.000", fps = 10.0)
+    # Can add int (frames), float (seconds), or str (timecode).
+    print(x + 10)
+    print(x + 10.0)
+    print(x + "00:10:00")
+    # Same for all comparison operators.
+    print((x + 10.0) == "00:01:10.000")
+
+
+:py:class:`FrameTimecode` objects can be added and subtracted, however the current implementation
+disallows negative values, and will clamp negative results to 0.
+
+.. warning::
+
+    Be careful when subtracting :py:class:`FrameTimecode` objects or adding negative
+    amounts of frames/seconds. In the example below, ``c`` will be at frame 0 since
+    ``b > a``, but ``d`` will be at frame 5:
+
+    .. code:: python
+
+        a = FrameTimecode(5, 10.0)
+        b = FrameTimecode(10, 10.0)
+        c = a - b   # b > a, so c == 0
+        d = b - a
+        assert(c == 0)
+        assert(d == 5)
+
 """
 
 import math
 from typing import Union
 
-MAX_FPS_DELTA = 1.0 / 100000
+MAX_FPS_DELTA: float = 1.0 / 100000
 """Maximum amount two framerates can differ by for equality testing."""
 
+# TODO(v0.6.2): Replace uses of Union[int, float, str] with TimecodeValue.
+TimecodeValue = Union[int, float, str]
+"""Named type for values representing timecodes. Must be in one of the following forms:
+
+  1. Timecode as `str` in the form 'HH:MM:SS[.nnn]' (`'01:23:45'` or `'01:23:45.678'`)
+  2. Number of seconds as `float`, or `str` in form 'Ss' or 'S.SSSs' (`'2s'` or `'2.3456s'`)
+  3. Exact number of frames as `int`, or `str` in form NNNNN (`123` or `'123'`)
+"""
+
 
 class FrameTimecode:
     """Object for frame-based timecodes, using the video framerate to compute back and
@@ -36,9 +89,9 @@ class FrameTimecode:
 
     A timecode is valid only if it complies with one of the following three types/formats:
 
-    1) Timecode as `str` in the form 'HH:MM:SS[.nnn]' (`'01:23:45'` or `'01:23:45.678'`)
-    2) Number of seconds as `float`, or `str` in form 'Ss' or 'S.SSSs' (`'2s'` or `'2.3456s'`)
-    3) Exact number of frames as `int`, or `str` in form NNNNN (`123` or `'123'`)
+      1. Timecode as `str` in the form 'HH:MM:SS[.nnn]' (`'01:23:45'` or `'01:23:45.678'`)
+      2. Number of seconds as `float`, or `str` in form 'Ss' or 'S.SSSs' (`'2s'` or `'2.3456s'`)
+      3. Exact number of frames as `int`, or `str` in form NNNNN (`123` or `'123'`)
     """
 
     def __init__(self,
